/*----------------------------------------------------------------+
* Authors: Lucio N. Lira, Silvio P. Lago.                        
* Institution: So Paulo Technological College (FATEC-SP).       
* Copyright (c) 2013-2016					  
*								  
* This file is part of aristotle software source code.           
* The software source code is distributed under MIT License.     
* Read LICENSE.txt for further information.			  
+-----------------------------------------------------------------+
 Plato - a system for dialogical argumentation                   
                                                                 
 Plato (424/423 BC  348/347 BC), was a Classical Greek          
 philosopher, mathematician, student of plato, writer of         
 philosophical dialogues, and founder of the Academy in Athens,  
 the first institution of higher learning in the Western world.  
 Along with his mentor, plato, and his student, Aristotle, Plato 
 helped to lay the foundations of Western philosophy and science.
 [http://thyselfknow.com/plato/]                                 
+----------------------------------------------------------------*/

:- module(plato, [plato/0]).

% ------------------------------------------------------------------
% Importao de mdulos
% ------------------------------------------------------------------

:- use_module(modules/fbrowser).
:- use_module(modules/updateKB).
:- use_module(modules/globvars).

% ------------------------------------------------------------------
% Definio de operadores
% ------------------------------------------------------------------

%:- export(':'/2).
:- op(100,  fy, not).      % Operador de negao.
:- op(101, xfy, and).      % Operador lgico de conjuno.
:- op(102, xfx, then).     % Operador de implicao.
:- op(103, xfx, :).        % Operador para rtulo de regras.
:- op(104, xfx, believes). % Operador que indica crena.
:- op(105, xfx, precedes). % Operador que estabelece precedncia.

% ------------------------------------------------------------------
% Definio de predicados dinmicos
% ------------------------------------------------------------------

:- dynamic aprende/1. % Define a possibilidade de aprendizado durante o dilogo.
:- dynamic choice/1.  % Guarda a precedncia mediada entre regras revogveis.
:- dynamic line/1.    % Insere as linhas do dilogo na memria.
:- dynamic explain/1. % Define a necessidade de explicao total no dilogo.
:- dynamic next/2.    % Define o prximo ato de fala permitido.
:- dynamic ptype/1.   % Define o tipo de relao de precedncia ativo.
:- dynamic tmp/1.     % Clusula para regras temporrias.
:- dynamic verbose/1. % Define se o dilogo ter sadas detalhadas.

:- discontiguous precedence_relation/2.

% ------------------------------------------------------------------
% Interface grfica
% ------------------------------------------------------------------

plato :-
    nb_setval(precedences, []),
    not(exists_frame),
    new(Frame,                       frame('Plato - Dialogical Argumentation (version 2.0.0)')),
    new(MenuBar,                     menu_bar),
    new(FileMenu,                    popup(file)),
    new(OptionsMenu,                 popup(options)),
    new(KnowledgebaseTab,            tab(knowledgebase)),
    new(DisplayTab,                  tab(display)),
    new(ArgumentsTab,                tab(arguments)),
    new(DialogueTab,                 tab(dialogue)),
    new(TabStack,                    tab_stack),
    new(KnowledgebaseEditor,         editor),
    new(ArgumentsEditor,             editor),
    new(DisplayEditor,               editor),
    new(DialogueEditor,              editor),
    new(RulePrecedenceMenu,          menu(rule_precedence, toggle)),
    new(Picture,                     picture),
    new(RightDialog,                 dialog),
    new(LeftDialog,                  dialog),
    new(Column_Left,                 dialog),
    var(plato:frame,                 Frame),
    var(plato:knowledgebase_tab,     KnowledgebaseTab),
    var(plato:arguments_tab,         ArgumentsTab),
    var(plato:display_tab,           DisplayTab),
    var(plato:dialogue_tab,          DialogueTab),
    var(plato:knowledgebase_editor,  KnowledgebaseEditor),
    var(plato:arguments_editor,      ArgumentsEditor),
    var(plato:display_editor,        DisplayEditor),
    var(plato:dialogue_editor,       DialogueEditor),
    var(plato:tab_stack,             TabStack),
    var(plato:rule_precedence_menu,  RulePrecedenceMenu),
    var(plato:picture,               Picture),
    var(plato:current_path,          @('')),
    new(ArgumentsButton,             button(arguments,           message(@prolog, arguments_click))),
    new(ConflictsButton,             button(conflicts,           message(@prolog, consist_prec))),
    new(DialogueButton,              button(dialogue,            message(@prolog, dialogue))),
    new(OpenMenuItem,                menu_item(open,             message(@prolog, loadfile, plato, dkb))),
    new(NewMenuItem,                 menu_item(new,              message(@prolog, newfile, plato, dkb))),
    new(SaveMenuItem,                menu_item(save,             message(@prolog, savefile, plato))),
    new(SaveAsMenuItem,              menu_item(save_as,          message(@prolog, saveasfile, plato, dkb), end_group := true)),
    new(QuitMenuItem,                menu_item(quit,             message(@prolog, close))),
    new(LearningMenuItem,            menu_item('Learning mode (off)',    message(@prolog, learning_mode))),
    new(ExplanationMenuItem,         menu_item('Explanation mode (off)', message(@prolog, explanation_mode))),
    new(VerboseMenuItem,             menu_item('Verbose mode (off)',     message(@prolog, verbose_mode),end_group := true)),
    var(plato:learning_menu_item,    LearningMenuItem),
    var(plato:explanation_menu_item, ExplanationMenuItem),
    var(plato:verbose_menu_item,     VerboseMenuItem),
    new(PrecedencesMenuItem,         menu_item(show_precedences, message(@prolog, show_precedences))),
    new(ItemExplicit,                menu_item(explicit,         message(@prolog, consist_prec))),
    new(ItemImplicit,                menu_item(implicit,         message(@prolog, consist_prec))),
    new(ItemMediated,                menu_item(mediated,         message(@prolog, consist_prec))),
    var(plato:item_explicit,         ItemExplicit),
    var(plato:item_implicit,         ItemImplicit),
    var(plato:item_mediated,         ItemMediated),
    send_list(Frame,                 append, [LeftDialog,RightDialog]),
    send_list(MenuBar,               append, [FileMenu,OptionsMenu]),
    send_list(FileMenu,              append, [OpenMenuItem,NewMenuItem,SaveMenuItem,SaveAsMenuItem,QuitMenuItem]),
    send_list(OptionsMenu,           append, [LearningMenuItem,ExplanationMenuItem,VerboseMenuItem,PrecedencesMenuItem]),
    send_list(TabStack,              append, [KnowledgebaseTab,DisplayTab,ArgumentsTab,DialogueTab]),
    send_list(LeftDialog,            append, [MenuBar,TabStack,ArgumentsButton,ConflictsButton,DialogueButton]),
    send_list(RightDialog,           append, [RulePrecedenceMenu]),
    send_list(RulePrecedenceMenu,    append, [ItemExplicit,ItemImplicit,ItemMediated]),
    forall(member(Tab/Editor, [KnowledgebaseTab/KnowledgebaseEditor,DisplayTab/DisplayEditor,
                               ArgumentsTab/ArgumentsEditor,DialogueTab/DialogueEditor]),
          (send(Tab, gap, size(2, 2)),
           send(Tab, append, Editor))),
    forall(member(Editor/Background, [KnowledgebaseEditor/aliceblue,DisplayEditor/lavenderblush,
                                      ArgumentsEditor/lightyellow,DialogueEditor/mintcream]),
          (send(Editor, size, size(70, 28)),
           send(Editor, font, font(screen, normal, 12)),
           send(Editor, colour, darkblue),
           send(Editor, background, Background))),
    send(LeftDialog, left, RightDialog),
    send(RightDialog, gap, size(15, 3)),
    send(RightDialog, append, Picture),
    send(Picture, size, size(480, 480)),
    send(Column_Left, pen, 0),
    send(Column_Left, gap,    size(0,0)),
    send(Column_Left, left,   LeftDialog),
    send(Column_Left, append, bitmap(image('resources/images/jpg/column_p.jpg'))),
    send(Frame, open, point(50, 70)).

% Funo: Gera todos os argumentos.
% Sadas: Mensagem confirmando a gerao de argumentos.
% Opes: Boto "Ok" para fechar a caixa de dilogo.

arguments_click :-
   var( plato:display_editor, DisplayEditor ),
   updateKB( plato ),
   %consist_prec(plato),
   ( get( DisplayEditor, selected, _ )
   ->  var( plato:display_tab, DisplayTab ),
       var( plato:tab_stack, TabStack ),
       send( TabStack, on_top, DisplayTab )
   ;   update_args,!, arguments_success ).

arguments_success :-
    not(exists_dialog),
    new(Dialog, dialog('Argumentational System')),
    send(Dialog, append, text('All arguments were successfully generated.')),
    send(Dialog, append, button(ok, message(Dialog, destroy))),
    var(plato:dialog, Dialog),
    send(Dialog, open, point(880, 280)).

% Verifica se o frame j existe (est em exibio).
exists_frame :-
    var(plato:frame, Frame),
    catch(send(Frame, expose), Error, true),
    var(Error).

% Verifica se o dialog j existe (est em exibio).
exists_dialog :-
    var(plato:dialog, Dialog),
    catch(send(Dialog, expose), Error, true),
    var(Error).

% Fecha a janela do mdulo plato.
close :-
    var(plato:frame, Frame),
    send(Frame, destroy).

% Funo: Verifica se as regras de precedncia explcita esto corretas.
% Sadas: (1) Mensagem indicando a existncia de ciclos.
%         (2) Mensagem indicando a verificao de consistncia.
%         (3) Mensagem indicando que os conflitos no podem ser gerados.
/*
consist_prec(Caller) :-
   var( Caller:display_editor, DisplayEditor ),
   var( Caller:display_tab, DisplayTab ),
   var( Caller:tab_stack, TabStack ),
   vrtices(Caller, Vs),
   findall(V->R,(member(V,Vs),acessa(Caller, V,ciclo(R))),L),
   findall(A,(member(T,L), term_to_atom(T,A)),P),
   P \= [],
   Separator='\n--------------------------------------------------------------------\n',
   atomics_to_string(['Error: cyclic precedence rules.',Separator],S),
   atomic_list_concat([S|P],Ciclos),
   send( TabStack, on_top, DisplayTab ),
   send(DisplayEditor,clear),
   send(DisplayEditor,append, Ciclos).

consist_prec(_) :- set_type, conflicts.
*/

consist_prec :- updateKB(plato),( consist_prec(plato) -> set_type, conflicts; true).

/*
vrtices(Caller, Vrtices) :-
    findall([V,W], Caller:precedes(V, W), L),
    flatten(L, F),
    sort(F, Vrtices).

acessa(Caller, V,L) :- acessa(Caller,[V], nil, [], L).

acessa(_, [], _, A, acessa(L)) :- reverse(A, L), !.

acessa(_, _, A, A, ciclo(L)) :- reverse(A, L), !.

acessa(Caller, [V|Vs], _, A, L) :-
    findall(W, Caller:precedes(V, W), F),
    union(Vs, F, N),
    union([V], A, M),
    acessa(Caller, N, A, M, L).
*/
% ------------------------------------------------------------------
% Gerao de argumentos e atualizaes
% ------------------------------------------------------------------

% arguments(+File): gera os argumentos a partir das regras.
% +File: define se as regras sero coletadas do arquivo (yes) ou,
% caso contrrio (no), da memria.
/*
arguments(Update) :-
    var(plato:display_editor, DisplayEditor),
    ( Update=yes -> updateKB(plato); true),
    (get(DisplayEditor, selected, _)
    ->  var(plato:display_tab, DisplayTab),
        var(plato:tab_stack, TabStack),
        send(TabStack, on_top, DisplayTab)
    ;   update_args).
*/
% Atualiza a visualizao dos argumentos.
update_args :-
    var(plato:arguments_editor, ArgumentsEditor),
    var(plato:arguments_tab, ArgumentsTab),
    var(plato:tab_stack, TabStack),
    send(ArgumentsEditor, clear),
    all_arguments(Args),
    forall(member(Arg, Args),
          (formatted(Arg, Text),
	   send(ArgumentsEditor, append, Text))),
    send(TabStack, on_top, ArgumentsTab),
    send(ArgumentsEditor, caret, 0).

% Formata um argumento como um texto.
formatted(Number:Support, Text) :-
    term_to_atom(Number, N),
    atom_length(N, Length),
    Tab is Length + 5,
    swritef(NewLine, ',\n%r', [' ', Tab]),
    join(NewLine, Support, Atoms),
    vert(Agent, Number),
    (Agent = paul -> A = 'P' ; A = 'O'),
    atomic_list_concat([A, N, ' : [' | Atoms], Text).

% Junta as clasulas em uma lista com uma nova linha.
join(_, [N: Last], [Atom,']\n\n']) :- !, term_to_atom(N:Last, Atom).

join(NewLine, [N:First|Rest], [Atom,NewLine|Atoms]) :-
    term_to_atom(N:First, Atom),
    join(NewLine, Rest, Atoms).

% Gera todos os argumentos possveis.
all_arguments(Arguments) :-
    retractall(vert(_,_)),
    retractall(edge(_,_)),
    flag(arg_number, _, 1),
    claims(Claims),
    findall(Order/Agent/Length/Support,
	   (member(Order/Agent, [1/paul, 2/olga]),
	    member(Fact, Claims),
	   (argument(Agent, Fact, Support)
	   ;argument(Agent, not(Fact), Support)),
	   length(Support, Length)),
	   Supports),
    msort(Supports, SortedSupports),
    findall(N:Support,
	   (member(Order/Agent/Length/Support, SortedSupports),
	    flag(arg_number, N, N + 1),
	    assertz(vert(Agent, N))),
	    Arguments),
    forall((member(N1:S1, Arguments),
	    member(N2:S2, Arguments),
	    N1 \= N2, attacks(S1, S2)),
	    assertz(edge(N1, N2))).

% Verifica se o suporte de S1 ataca o suporte de S2.
attacks(Support1, Support2) :-
    last(Support1, Rule1:then(_, Fact1)),
    member(Rule2:then(_C, Fact2), Support2),
    neg(Fact1, Fact2),
    not(prec(Rule2, Rule1)), !.

% Gera todas as locues possveis.
claims(Claims) :-
    predicates(Predicates),
    objects(Objects),
    findall(Claim, (member(Predicate/Arity, Predicates),
		    length(Arguments, Arity),
		    tuple(Arguments, Objects),
		    Claim =.. [Predicate|Arguments]),
	    Claims).

% Localiza todos os objetos relevantes na base de conhecimento.
objects(Objects) :-
    findall(Object, (believes(_,then(_,_,Conclusion)),
		    (Conclusion = not(Fact) -> Fact =.. [_|Arguments]
		    ;Conclusion =.. [_|Arguments]),
		     member(Object, Arguments),
		     atomic(Object)),
	    ObjectList),
    sort(ObjectList, Objects).

% Localiza todos os predicados relevantes na base de conhecimento.
predicates(Predicates) :-
    findall(Predicate/Arity, (believes(_,then(_,_,Conclusion)),
			      (Conclusion = not(Fact) -> Fact =.. [Predicate|Arguments]
			      ; Conclusion =.. [Predicate|Arguments]),
			      length(Arguments, Arity)),
	    PredicatetList),
    sort(PredicatetList, Predicates).

% Gera uma tupla com os objetos de um conjunto.
tuple([], _).

tuple([Object|Tuple], Set) :-
    member(Object, Set),
    tuple(Tuple, Set).

% Gera um argumento para um fato (i.e., um suporte).
argument(Agent, Fact, Support) :- argument(Agent, Fact and true, [], Support).

argument(_Agent, true, Support, Support).

argument(Agent, Fact and Facts, Partial, Support) :-
    believes(Agent, then(RuleNumber, Conditions, Fact)),
    add(Conditions, Facts, NewFacts),
    union([RuleNumber : Conditions then Fact], Partial, NewPartial),
    argument(Agent, NewFacts, NewPartial, Support).

% Anexa duas frmulas conjuntivas.
add(true, B, B) :- !.
add(A and B, C, A and D) :- add(B, C, D).
add(A, B, A and B).

%+-----------------------------------------------------------------+
%                  GRAFO DE DECISO DE COERNCIA                  
% amarelo (bloqueado), vermelho(derrotado) e verde(no derrotado) 
%+-----------------------------------------------------------------+

% Exibe o grafo de deciso de coerncia dos argumentos da base de
% conhecimentos.
conflicts :-
    update_args,
    var(plato:picture, Picture),
    send(Picture, clear),
    draw_vertices,
    draw_links.

pconflicts :-
    var(plato:picture, Picture),
    send(Picture, clear),
    draw_vertices,
    draw_links.

% Desenha os vrtices.
draw_vertices :-
    initiate_position_variables,
    retractall(vertex(_, _)),
    forall(vert(_, Label),
	  (draw_vertex(Label, ObjectId),
	   assertz(vertex(Label, ObjectId)))),
    colour_vertices.

% Inicializao da posio global das variveis.
initiate_position_variables :-
    findall(Vertex, vert(_, Vertex), Vertices),
    length(Vertices, Length),
    Height is 75 + (ceil(sqrt(Length) -1)) * 75,
    flag(current_x, _, 50),
    flag(current_y, _, 50),
    flag(picture_h, _, Height).

% Desenha um nico vrtice e retorna seu identificador.
draw_vertex(Label, ObjectId) :-
    var(plato:picture, Picture),
    get_position(X, Y),
    (vert(paul,Label) -> V = box(25,25) ; V = circle(25)),
    send(Picture, display, new(ObjectId, V), point(X,Y)),
    send(Picture, display, new(Text, text(Label)), point(X,Y)),
    send(Text, font, font(screen ,bold, 11)),
    new(_, constraint(ObjectId, Text, identity(center))),
    send(ObjectId, handle, handle(w/2,0,up)),
    send(ObjectId, handle, handle(w/2,h,dn)),
    send(ObjectId, handle, handle(0,h/2,lt)),
    send(ObjectId, handle, handle(w,h/2,rt)),
    send(ObjectId, recogniser, new(move_gesture(left))).

% Retorna a posio onde o prximo vrtice ser desenhado.
get_position(X, Y) :-
    flag(picture_h, H, H),
    flag(current_x, X, X),
    flag(current_y, Y, Y),
    (Y+75 < H -> flag(current_y, _, Y+75)
    ;flag(current_x, _, X+75),
     flag(current_y, _, 50)).

% Desenha as arestas entre os vrtices.
draw_links :-
    forall(edge(Label1,Label2),
	  (vertex(Label1,Vertex1),
	   vertex(Label2,Vertex2),
	   handles(Vertex1, Vertex2, Out, In),
	   new(Link, link(Out, In, line(arrows := second))),
	   send(Vertex1, connect, Vertex2, Link))).

% Define qual o formato da aresta entre dois vrtices.
handles(Vertex1, Vertex2, Out, In) :-
    get(Vertex1, position, point(X1,Y1)),
    get(Vertex2, position, point(X2,Y2)),
    DeltaX is X2 - X1,
    DeltaY is Y2 - Y1,
    (DeltaX = 0 -> (DeltaY > 0 -> Out = dn, In = up ; Out = up, In = dn)
    ;DeltaY = 0 -> (DeltaX > 0 -> Out = rt, In = lt ; Out = lt, In = rt)
    ;DeltaX > 0 -> (DeltaY > 0 -> Out = dn, In = up ; Out = up, In = dn)
    ;DeltaY > 0 -> (DeltaX > 0 -> Out = up, In = dn ; Out = dn, In = up)
    ;DeltaX < 0 -> (DeltaY > 0 -> Out = dn, In = up ; Out = up, In = dn)
    ;Out=dn, In=up).

% Colore os vrtices para indicar a coerncia dos argumentos.
colour_vertices :-
    findall(V/Vo, vertex(V,Vo), Vs),
    colour_vertices(Vs).

colour_vertices([]) :-
    forall((vert(olga, V), vertex(V,Vo)),
	   (get(Vo, fill_pattern, colour(honeydew2)) -> send(Vo, fill_pattern, colour(palegreen))
	   ;get(Vo, fill_pattern, colour(lightpink)   ) -> send(Vo, fill_pattern, colour(salmon))
	   ;send(Vo, fill_pattern, colour(khaki)) )), !.

colour_vertices(Vs) :-
    select(W/Wo, Vs, Vss),
    (edge(V,W), vertex(V,Vo), get(Vo, fill_pattern, colour(honeydew2)), !, C = lightpink
    ;not((edge(V,W), vertex(V,Vo), get(Vo, fill_pattern, @nil))), !, C = honeydew2),
    send(Wo, fill_pattern, colour(C)), !,
    colour_vertices(Vss).

colour_vertices(Vs) :-
   forall(member(_/Wo, Vs),
	   send(Wo, fill_pattern, colour(cornsilk))),
   colour_vertices([]).

% ------------------------------------------------------------------
% Relao de precedncia mediada
% ------------------------------------------------------------------

% precedence_relation(+mediated,-Precedences): verdade se
% Precedences  um conjunto de regras de precedncia mediadas
% pelo usurio partindo dos conflitos da base de conhecimentos.
precedence_relation(mediated, Precedences) :-
    findall(L1<L2,
	    (believes(_, then(L1, _, C1)),
	     believes(_, then(L2, _, C2)),
	     L1 \= L2,
	     neg(C1, C2)),
	    Conflicts1),
    nb_getval(precedences, ActivePrecedences1),
    reverse_pairs(ActivePrecedences1, ActivePrecedences2),
    subtract(Conflicts1, ActivePrecedences1, Conflicts2),
    subtract(Conflicts2, ActivePrecedences2, Conflicts),
    escolhe(Conflicts, [], Escolhidas),
    union(ActivePrecedences1, Escolhidas, Precedences1),
    transitive_closure(Precedences1, Closure),
    filtered(Closure, Precedences).

% reverse_pair(?A,?B): gera B com a inverso dos pares de A.
% ?A: Lista com relaes de precedncia.
% ?B: Lista com relaes de precedncia.
reverse_pairs([], []) :- !.
reverse_pairs([(X<Y)|Resto1], [(Y<X)|Resto2]) :- reverse_pairs(Resto1, Resto2).

% escolhe(+O, +A, -E): gera E a partir das opes escolhidas.
% +O: lista de opes de regras conflitantes passveis de mediao.
% +A: acumulador de regras de precedncia escolhidas.
% -E: lista final com as regras de precedncia mediadas.
escolhe([], Acumulador, Escolhidas) :- !, reverse(Acumulador, Escolhidas).

escolhe([(L1<L2)|Opcoes], Acumulador, Escolhidas) :-
    ((acyclic([L1<L2|Acumulador]), acyclic([L2<L1|Acumulador]))
    -> choose([L1,L2])
    ;  true),
    repeat, sleep(1), retract(choice(L)), !,
    (L = L1 -> NovoAcumulador = [(L1<L2)|Acumulador]
    ;L = L2 -> NovoAcumulador = [(L2<L1)|Acumulador]
    ;NovoAcumulador = Acumulador),
    subtract(Opcoes, [(L1<L2),(L2<L1)], NovaOpcoes),
    escolhe(NovaOpcoes, NovoAcumulador, Escolhidas).

% choose(+[L1,L2]): Abre uma janela para escolha de uma precedncia.
% +[L1,L2]: lista com um par de rtulos de regras conflitantes.
choose([L1,L2]) :-
    believes(_, then(L1, A1, C1)),
    believes(_, then(L2, A2, C2)),
    term_to_atom(L1:then(A1, C1), Atom1),
    term_to_atom(L2:then(A2, C2), Atom2),
    new(D, dialog('Conflict Mediator')),
    send(D, append, new(C, menu(choose, marked))),
    send_list(C, append, [Atom1,Atom2,none]),
    send(C, layout, vertical),
    send(D, append, button(ok, and(message(@prolog, set_choice, C?selection),
                                   message(D, destroy)))),
    send(D, default_button, ok),
    send(D, open, point(880, 280)).

set_choice(Choice) :-
	(Choice = none
	-> Label = none
	;atom_to_term(Choice, Label:_,_)),
	assert(choice(Label)).

% ------------------------------------------------------------------
% Predicados auxiliares
% ------------------------------------------------------------------

% unflawed(+Clauses): verdade se todas as regras revogveis
% na lista Clauses so consistentes.
unflawed([]) :- !.

unflawed([precedes(_,_)|Clauses]) :- !, unflawed(Clauses).

unflawed([_ believes _ : then(A,C)|Clauses]) :-
    consistent(then(A,C)), !,
    unflawed(Clauses).

unflawed([Agent believes L : then(A,C)|_]) :-
    ansi_format([bold,fg(black)],'~nError: ', []),
    format('inconsistent rule (~w)~n~n', [Agent believes L : then(A,C)]),
    !, fail.

% consistent(+then(Antecedent, Consequent)): verdade se no existe um
% par de literais complementares no conjunto de literais formado pelos
% literais usados no antecedente mais o literal usado no consequente.
% a(b, c, d)        --> (a, b, c, d) ok
% a(b, c, d, not d) --> (a, b, c, d, not d) erro!
% a(b, c, d, not a) --> (a, b, c, d, not a) erro!
consistent(then(Antecedent, Consequent)) :-
    findall(Premise, in(Premise, Antecedent), Premises),
    not((member(Proposition1, [Consequent|Premises]),
         member(Proposition2, [Consequent|Premises]),
         neg(Proposition1, Proposition2))), !.

% gratuitous(+Clauses): verdade se Clauses tem uma regra com um
% consequente gratuito (e.g., 'a then a', ou 'p then q' e 'q then p').
gratuitous(Clauses) :-
    findall(R, (member(R, Clauses), gratuitous(R, Clauses)), G), G \= [],
    ansi_format([bold,fg(black)],'~nWarning: ', []),
    format('gratuitous consequents (~w)~n~n', [G]).

gratuitous(_ believes _ : then(A, C), _) :- in(C, A), !.

gratuitous(_ believes _ : then(A1, C1), Clauses) :-
    member(_ believes _ : then(A2, C2), Clauses),
    in(L1, A1), der(L1, C2),
    in(L2, A2), der(L2, C1), !.

% rule(+L: A then C): verdade se existe a regra de rtulo L, com
% antecedente A e consequente C, na base de conhecimentos.
rule(Label: Antecedent then Consequent) :-
    believes(_, then(Label, Antecedent, Consequent)).

% in(?L, +C): verdade se L  um literal na conjuno C.
in(L, L) :- L \= true, L \= and(_, _).
in(L, and(L, _)).
in(L, and(_, C)) :- in(L, C).

% sset(+A, +B): verdade se cada literal na conjuno A  tambm
% um literal na conjuno B, ou seja, A  um subconjunto de B.
sset(A, B) :- forall(in(X, A), in(X, B)).

% Define a relao de precedncia que ser usada.
set_type :-
    var(plato:rule_precedence_menu, RulePrecedenceMenu),
    var(plato:item_explicit, @E),
    var(plato:item_implicit, @I),
    get(RulePrecedenceMenu, selected, explicit, Explicit),
    get(RulePrecedenceMenu, selected, implicit, Implicit),
    get(RulePrecedenceMenu, selected, mediated, Mediated),
    (Explicit = @on,  Implicit = @off, !, def_prec(explicit), nb_getval(explicit, V)
    ;Explicit = @off, Implicit = @on,  !, def_prec(implicit), nb_getval(implicit, V)
    ;Explicit = @on,  Implicit = @on,  !, def_prec(mixed),    nb_getval(mixed,    V)
    ;Explicit = @off, Implicit = @off, !, def_prec(none),     V = []),
    nb_setval(precedences, V),
    (Mediated = @on
    -> (send(@E, active, @off),
        send(@I, active, @off),
        def_prec(mediated),
        precedence_relation(mediated, W),
        nb_setval(mediated, W),
	nb_setval(precedences, W))
    ;  (send(@E, active, @on),
        send(@I, active, @on),
        nb_setval(mediated, []))).

% Define que no ser usada relao de precedncia.
:- retractall(ptype(_)), assert(ptype(none)).

% Define que no ser permitido o aprendizado no dilogo.
:- retractall(aprende(_)), assert(aprende(no)).

% Define que toda alegao do oponente deve ser justificada.
:- retractall(explain(_)), assert(explain(no)).

% Define o modo de dilogo como no detalhado.
:- retractall(verbose(_)), assert(verbose(no)).

% Define a relao de precedncia que ser usada.
def_prec(Precedence) :- retractall(ptype(_)), assert(ptype(Precedence)).

% Define a possibilidade de aprendizado no dilogo.
learning_mode :- retract(aprende(no)), assert(aprende(yes)),
	modify_name(plato:learning_menu_item, 'Learning mode (on)'), !.

learning_mode :- retract(aprende(yes)), assert(aprende(no)),
	modify_name(plato:learning_menu_item, 'Learning mode (off)'), !.

% Define a possibilidade de aceitar alegaes j conhecidas sem explicaes.
explanation_mode :- retract(explain(no)), assert(explain(yes)),
	modify_name(plato:explanation_menu_item, 'Explanation mode (on)'), !.

explanation_mode :- retract(explain(yes)), assert(explain(no)),
	modify_name(plato:explanation_menu_item, 'Explanation mode (off)'), !.

% Define o modo de dilogo como detalhado ou no.
verbose_mode :- retract(verbose(no)), assert(verbose(yes)),
	modify_name(plato:verbose_menu_item, 'Verbose mode (on)'), !.

verbose_mode :- retract(verbose(yes)), assert(verbose(no)),
	modify_name(plato:verbose_menu_item, 'Verbose mode (off)'), !.

% modify_name(+C, +N): altera o nome de C para N.
% +C: controle
% +N: novo nome
modify_name(Control, Name) :- var(Control, C), send(C, value, Name).

% Exibe todas as possveis precedncias entre regras revogveis.
show_precedences :-
   var(plato:display_editor, DisplayEditor ),
   var(plato:display_tab, DisplayTab ),
   var(plato:tab_stack, TabStack ),
   send(TabStack, on_top, DisplayTab ),
   nb_getval(explicit, Explicit), term_to_atom(Explicit, E),
   nb_getval(implicit, Implicit), term_to_atom(Implicit, I),
   nb_getval(mixed,    Mixed),    term_to_atom(Mixed, M),
   nb_getval(mediated, Mediated), term_to_atom(Mediated, Me),
   Separator = '\n--------------------------------------------------------------------\n',
   send(DisplayEditor, clear),
   atomic_list_concat(['Precedence Relations',Separator], Title),
   send(DisplayEditor, append, Title),
   atom_concat('Explicit: ',   E, EText),   send(DisplayEditor, append, EText),
   atom_concat('\nImplicit: ', I, IText),   send(DisplayEditor, append, IText),
   atom_concat('\nMixed:    ', M, MText),   send(DisplayEditor, append, MText),
   atom_concat('\nMediated  ', Me, MeText), send(DisplayEditor, append, MeText),
   send(DisplayEditor, append, Separator),
   ptype(SelectedType),
   atomic_list_concat(['Currently using: ',SelectedType,Separator], CText),
   send(DisplayEditor, append, CText).

%+------------------------------------------------------------------+
%                        DILOGO PERSUASIVO                        
%          Dois agentes no dilogo (proponente e oponente)         
%+------------------------------------------------------------------+

% start_dialogue(+L): inicia o dilogo persuasivo cujo objetivo
%  provar que L  coerente.
% +L: literal que dever ser defendido
start_dialogue(Literal) :-
    var(plato:dialogue_editor, DialogueEditor),
    send(DialogueEditor, clear),
    flag(arg, _, 1),
    retractall(next(start, _)),
    asserta(next(start, paul:claim(Literal))),
    nb_setval(narrative, []),
    ignore(dialogue(start)),
    outcome(Literal, Outcome, C),
    term_to_atom(Literal, L),
    atomic_list_concat(['\n', Outcome, ': ', L], Result),
    assertz(line(Result)),
    send(DialogueEditor?image, colour, C), !.

% ------------------------------------------------------------------
% Dilogo persuasivo e incio da locuo
% ------------------------------------------------------------------

% Abre uma janela para a insero do literal que ser defendido.
dialogue :-
    updateKB(plato),
    ( exists_dialog -> var(plato:dialog, D), send(D,destroy) ; true),
    new(D, dialog('Persuasion Dialog')),
    new(T, text_item('Insert Claim')),
    send(D, append, T),
    send(D, append, button(show, message(@prolog, show, T?selection))),
    send(D, append, button(clear, message(T, clear))),
    var(plato:dialog, D),
    send(D, open, point(880, 280)).

% dialogue(+Act): simula um dilogo persuasivo entre paul e olga
% +Act: para comear o dilogo dever ser 'start'.
dialogue(Act) :-
    next(Act, NextAct),
    new(NextAct),
    say(NextAct),
    upd(NextAct),
    not(final(NextAct)),
    not(dialogue(NextAct)).

claim(C):-
    get(C, value, Text),
    read_from_chars(Text, Proposition),
    start_dialogue(Proposition).

% Exibe todo o dilogo de uma s vez
show(C) :-
    get(C, value, Text),
    read_from_chars(Text, Literal),
    flag(first_time, V, 0),
    (V = 1 -> start_dialogue(Literal)),
    fail.

show(_) :-
    var(plato:dialogue_editor, DialogueEditor),
    flag(index, _, 0),
    forall(line(Text), (send(DialogueEditor, append, Text), retract(line(Text)))),
    flag(first_time, _, 1).

show(_) :-
    var(plato:dialogue_editor, DialogueEditor),
    send(DialogueEditor, clear).

% new(+Agent:Locution): verdade se a locuo  indita na narrativa.
% Obs.: Somente o agente 'olga' pode repetir locues.
new(Agent:Locution) :-
    (Agent = olga, Locution = since(_)
    ;nb_getval(narrative, Narrative),
    not(memberchk(_:Locution, Narrative))).

% say(+Agent:Locution): exibe a locuo do agente.
say(Agent:Locution) :-
    var(plato:dialogue_editor, DialogueEditor),
    var(plato:dialogue_tab, DialogueTab),
    var(plato:tab_stack, TabStack),
    (  verbose(yes)
    -> flag(arg, N, N + 1),
       term_to_atom(Locution, AtomicLocution),
       atomic_list_concat(['<', N, '> ', Agent, ' : ', AtomicLocution, '\n'], Text),
       assertz(line(Text))
    ;  true),
    send(TabStack, on_top, DialogueTab),
    send(DialogueEditor, caret, 0).

% upd(+Agent:Locution): atualiza a narrativa com a locuo do agente.
upd(Agent:Locution) :-
    nb_getval(narrative, Narrative),
    nb_setval(narrative, [Agent:Locution|Narrative]).

% ------------------------------------------------------------------
% Regras de protocolo
% ------------------------------------------------------------------

% next(+L1, +L2): verdade se L1  uma locuo com L2 como contra
% movimento permitido.
next(A:claim(F), B:why(F))     :- adv(A, B), (explain(no), suspicion(B, F); explain(yes)).
next(A:claim(F), B:agree(F))   :- adv(A, B).
next(A:since(R), B:why(P))     :- adv(A, B), premise(P, R), (explain(no), suspicion(B, P); explain(yes)).
next(A:since(R), B:agree(P))   :- adv(A, B), premise(P, R), (explain(no), not(suspicion(B, P)); explain(yes)).
next(A:since(R), B:since(S))   :- adv(A, B), attacks(B, S, R).
next(A:since(R), B:agree(F))   :- adv(A, B), claim(R, F).
next(A:since(R), B:learn(F))   :- adv(A, B), aprende(yes), claim(R, F), learn(B, F).
next(A:why(F),   B:since(R))   :- adv(A, B), adduces(B, R, F).
next(A:why(F),   B:retract(F)) :- adv(A, B).

% final(+L): verdade se L  uma locuo de finalizao.
% +L: locuo
final(_:agree(_)).
final(_:learn(_)).
final(_:retract(_)).

% adv(?A,?B): verdade se A e B so adversrios.
% ?A: agente
% ?B: agente
adv(paul, olga).
adv(olga, paul).

% suspicion(+A, +C): verdade se A tem motivos para suspeitar de C.
% +A: agente
% +C: consequente
suspicion(Agent, Consequent) :-
    not(argument(Agent, Consequent, _));
    neg(Consequent, Oposto),
    argument(Agent, Oposto, A),
    forall(argument(Agent, Consequent, B),
              (last(A, L1:_), last(B, L2:_),
               not(prec(L2, L1)))).

% learn(+A, +C): verdade se A no tem um argumento para C.
% +A: agente
% +C: consequente
learn(A, C) :-
    not(argument(A, C, _)),
    updateKB:insertKB(believes(A, then(true, C))),
    conflicts.

% outcome(+L, -O, -C): define o resultado do dilogo e a cor
% que ser exibida.
% +L: literal
% -O: resultado
% -C: cor
outcome(Literal, Outcome, Color) :-
    nb_getval(narrative, Narrative),
    Narrative \= [],
    (member(olga:agree(Literal), Narrative)
    -> (Outcome, Color) = ('Accepted', blue)
    ;  (Outcome, Color) = ('Rejected', red)).

% neg(?L, ?C): verdade se L e C so complementares.
% ?L: literal
% ?C: literal
neg(not(Proposition), Proposition) :- !.
neg(Proposition, not(Proposition)).

% coherent(+A then +C): verdade se A alegar C  coerente com a
% narrativa atual.
% +A: agente
% +C: consequente
coherent(A then C) :-
    nb_getval(narrative, Narrative),
    forall(in(L, C and A), not(member(_:retract(L), Narrative))),
    forall(in(L, C and A), not((neg(L, N), member(_:agree(N), Narrative)))),
    forall(in(L, A), not((member(_:why(L), Narrative), not(member(_:agree(L), Narrative))))),
    forall(in(L, A), not((neg(L,N), member(_:why(N), Narrative), not(member(_:retract(N), Narrative))))).

% adduces(+Agent, +R, +C): verdade se Agent pode alegar R com C.
% +Agent: agente
% +R:     regra revogvel
% +C:	  consequente da regra revogvel
adduces(Agent, R: A then C, C) :-
    believes(Agent, then(R, A, C)),
    coherent(A then C).

% premise(-L, +R): verdade se L  uma premissa de R.
% -L: literal
% +R: regra revogvel
premise(Literal, _:Antecedent then _) :-
    in(Literal, Antecedent),
    neg(Literal, Complement),
    coherent(Complement then nil).

% attacks(+A, +R1, +R2): verdade se A pode usar R1 para atacar R2.
% +A:  agente
% +R1: rtulo de uma regra revogvel
% +R2: rtulo de uma regra revogvel
attacks(Agent, R1: C then N, R2: _ then F) :-
    neg(F, N),
    believes(Agent, then(R1, C, N)),
    coherent(C then N),
    defeats(Agent, R1, R2).

% defeats(+A, +R1, +R2): verdade se R1 derrota R2.
% +A:  agente
% +R1: rtulo de uma regra revogvel
% +R2: rtulo de uma regra revogvel
defeats(paul, R1, R2) :- prec(R1, R2), !.
defeats(olga, R1, R2) :- prec(R1, R2), !.
defeats(olga, R1, R2) :- not(prec(R1, R2)), not(prec(R2, R1)).

% prec(+R1, +R2): verdade se R1 precede R2.
% +R1: rtulo de uma regra revogvel
% +R2: rtulo de uma regra revogvel
prec(R1, R2) :-
    nb_getval(precedences, Precedences),
    memberchk(R1<R2, Precedences).

% claim(+R, -C): verdade se h regra revogvel com consequente C.
% +R: regra revogvel
% -C: consequente da regra revogvel
claim(_ : _ then C, C).


















